# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2018-02-16 09:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0004_auto_20180216_0935'),
    ]

    operations = [
        migrations.AlterField(
            model_name='modelchangelog',
            name='information',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='automated_logging.ModelObject'),
        ),
    ]



"""
    This files handles processig all the request related Signals - originating from Django.
"""
import logging
from logging import CRITICAL, WARNING

from .. import settings
from django.dispatch import receiver
from django.core.handlers.wsgi import WSGIRequest
from django.core.signals import request_finished, got_request_exception
from . import get_current_environ, get_current_user


@receiver(request_finished, weak=False)
def request_finished_callback(sender, **kwargs):
    """ This function logs if the user acceses the page """
    logger = logging.getLogger(__name__)
    level = settings.AUTOMATED_LOGGING['loglevel']['request']

    user = get_current_user()
    uri, application, method = get_current_environ()

    logger.log(level, ('%s performed request at %s' % (user, uri)).replace("  ", " "), extra={
        'action': 'request',
        'data': {
            'user': user,
            'uri': uri,
            'method': method,
            'application': application
        }
    })


@receiver(got_request_exception, weak=False)
def request_exception(sender, request, **kwargs):
    """
        Automated request exception logging
        The function can also return an WSGIRequest exception,
        which does not supply either status_code or reason_phrase
    """

    if not isinstance(request, WSGIRequest):
        logger = logging.getLogger(__name__)
        level = CRITICAL if request.status_code <= 500 else WARNING

        logger.log(level, '%s exception occured (%s)', request.status_code, request.reason_phrase)

    else:
        logger = logging.getLogger(__name__)
        logger.log(WARNING, 'WSGIResponse exception occured')



"""
    file containing the required handlers for logging to the database

    this is just basic sorcery
"""
from logging import Handler


class DatabaseHandler(Handler):
    """Handler for logging into any database"""
    def __init__(self):
        super(DatabaseHandler, self).__init__()

    def emit(self, record):
        # add to database
        # try - except -> preventing circular import
        # http://stackoverflow.com/questions/4379042/django-circular-model-import-issue

        try:
            from .models import Model, Application, ModelObject, ModelChangelog
            from django.contrib.contenttypes.models import ContentType

            if hasattr(record, 'action'):
                if record.action == 'model':
                    if 'al_evt' in record.data['instance'].__dict__.keys():
                        entry = record.data['instance'].al_evt
                    else:
                        entry = Model()
                        entry.user = record.data['user']
                        entry.application = Application.objects.get_or_create(name=record.data['instance']._meta.app_label)[0]

                        entry.message = record.message

                        if record.data['status'] == 'add':
                            status = 1
                        elif record.data['status'] == 'change':
                            status = 2
                        elif record.data['status'] == 'delete':
                            status = 3
                        else:
                            status = 0

                        entry.action = status
                        entry.information = ModelObject()
                        entry.information.value = repr(record.data['instance'])
                        ct = ContentType.objects.get_for_model(record.data['instance'])

                        try:
                            assert_ct = ContentType.objects.get(pk=ct.pk)
                            ct_exists = True
                        except ContentType.DoesNotExist:
                            ct_exists = False

                        if ct_exists:
                            entry.information.type = ct
                        entry.information.save()

                        record.data['instance'].al_evt = entry

                    if record.data['status'] == 'modified' and 'al_chl' in record.data['instance'].__dict__.keys():
                        entry.modification = record.data['instance'].al_chl

                    entry.save()

                elif record.action == 'request':
                    from .models import Request

                    entry = Request()
                    entry.application = record.data['application']
                    entry.uri = record.data['uri']
                    entry.user = record.data['user']
                    entry.method = record.data['method']
                    entry.save()

            else:
                from .settings import AUTOMATED_LOGGING

                signal = True
                for excluded in AUTOMATED_LOGGING['exclude']:
                    if record.module.startswith(excluded):
                        signal = False
                        break

                if 'unspecified' not in AUTOMATED_LOGGING['modules']:
                    signal = False

                if signal:
                    from .models import Unspecified

                    entry = Unspecified()

                    if hasattr(record, 'message'):
                        entry.message = record.message

                    entry.level = record.levelno
                    entry.file = record.pathname
                    entry.line = record.lineno

                    entry.save()
        except Exception as e:
            print(e)
            pass



from django.conf import settings
import uuid
from django.db import models
from django.contrib.contenttypes.models import ContentType


class BaseModel(models.Model):
    id = models.UUIDField(default=uuid.uuid4, primary_key=True)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Application(BaseModel):
    name = models.CharField(max_length=255)

    def __str__(self):
        return self.name


class ModelStorage(BaseModel):
    name = models.CharField(max_length=255)

    def __str__(self):
        return self.name


class Field(BaseModel):
    name = models.CharField(max_length=255)
    model = models.ForeignKey(ModelStorage, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return '{} - {}'.format(self.name, self.model)


class ModelObject(BaseModel):
    value = models.CharField(max_length=255, null=True)
    field = models.ForeignKey(Field, null=True, on_delete=models.CASCADE)
    type = models.ForeignKey(ContentType, null=True, blank=True, on_delete=models.CASCADE, related_name='atl_modelobject_application')

    def __str__(self):
        output = '{}'.format(self.value)

        if self.field is not None:
            output += " of field {}".format(self.field)

        if self.type is not None:
            output += " in model {}.{}".format(self.type.app_label, self.type.model)

        return output


class ModelModification(BaseModel):
    previously = models.ManyToManyField(ModelObject, related_name='changelog_previous')
    currently = models.ManyToManyField(ModelObject, related_name='changelog_current')

    def __str__(self):
        print([str(v) for v in self.previously.all()])
        return ' {0} changed to {1}; '.format(", ".join(str(v) for v in self.previously.all()),
                                              ", ".join(str(v) for v in self.currently.all()))


class ModelChangelog(BaseModel):
    modification = models.OneToOneField(ModelModification, null=True, on_delete=models.CASCADE)
    inserted = models.ManyToManyField(ModelObject, related_name='changelog_inserted')
    removed = models.ManyToManyField(ModelObject, related_name='changelog_removed')

    information = models.OneToOneField(ModelObject, null=True, on_delete=models.CASCADE)

    def __str__(self):
        output = ''
        if self.modification is not None:
            output += '{0} in {1}'.format(self.modification, self.information)
        if self.inserted.count() > 0:
            output += '{0} was inserted into {1}; '.format(", ".join(str(v) for v in self.inserted.all()), self.information)
        if self.removed.count() > 0:
            output += '{0} was removed from {1}; '.format(", ".join(str(v) for v in self.removed.all()), self.information)

        return output


class Model(BaseModel):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)
    application = models.ForeignKey(Application, on_delete=models.CASCADE, related_name='atl_model_application', null=True)

    message = models.TextField(null=True)

    MODES = (
        (0, 'n/a'),
        (1, 'add'),
        (2, 'change'),
        (3, 'delete'),
    )
    action = models.PositiveSmallIntegerField(choices=MODES, default=0)

    information = models.OneToOneField(ModelObject, on_delete=models.CASCADE, related_name='atl_model_information', null=True)
    modification = models.ForeignKey(ModelChangelog, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return '{0} - {1}'.format(self.created_at, self.message)

    class Meta:
        verbose_name = 'Changelog'
        verbose_name_plural = 'Changelogs'


class Request(BaseModel):
    application = models.ForeignKey(Application, on_delete=models.CASCADE, null=True)

    uri = models.URLField()
    method = models.CharField(max_length=64)

    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return '{2} - {0} performed request at {3} ({1})'.format(self.user, self.created_at, self.uri, self.application)

    class Meta:
        verbose_name = "Request"
        verbose_name_plural = "Requests"


class Unspecified(BaseModel):
    message = models.TextField(null=True)
    level = models.PositiveSmallIntegerField(default=20)

    file = models.CharField(max_length=255, null=True)
    line = models.PositiveIntegerField(null=True)

    def __str__(self):
        if self.level == 10:
            level = 'DEBUG'
        elif self.level == 20:
            level = 'INFO'
        elif self.level == 30:
            level = 'WARNING'
        elif self.level == 40:
            level = 'ERROR'
        elif self.level == 50:
            level = 'CRITICAL'
        else:
            level = 'NOTSET'

        return '{} - {} - {} ({} - {})'.format(self.created_at, level, self.message, self.file, self.line)

    class Meta:
        verbose_name = "Non DJL Message"
        verbose_name_plural = "Non DJL Messages"


class LDAP(BaseModel):

    class Meta:
        verbose_name = "LDAP event log entry"
        verbose_name_plural = "LDAP event log entries"

    action = models.TextField()
    succeeded = models.NullBooleanField(blank=True, null=True)
    errorMessage = models.TextField(blank=True, null=True)

    basedn = models.TextField(blank=True, null=True)
    entry = models.TextField(blank=True, null=True)

    objectClass = models.TextField(blank=True, null=True)
    cn = models.TextField(blank=True, null=True)

    existing_members = models.TextField(blank=True, null=True)
    data_members = models.TextField(blank=True, null=True)
    diff_members = models.TextField(blank=True, null=True)


# from .signals import request, database, m2m
